@@ -508,7 +508,8 @@ cm_Analyze(cm_conn_t *connp,
                     if (code == 0)
                         free_svr_list = 1;
                 }
-                cm_ResetServerBusyStatus(volServerspp);
+                if (volServerspp)
+                    cm_ResetServerBusyStatus(volServerspp);
                 if (free_svr_list) {
                     cm_FreeServerList(volServerspp, 0);
                     free_svr_list = 0;
@@ -559,7 +560,8 @@ cm_Analyze(cm_conn_t *connp,
                     if (code == 0)
                         free_svr_list = 1;
                 }
-                cm_ResetServerBusyStatus(volServerspp);
+                if (volServerspp)
+                    cm_ResetServerBusyStatus(volServerspp);
                 if (free_svr_list) {
                     cm_FreeServerList(volServerspp, 0);
                     free_svr_list = 0;
@@ -651,7 +653,8 @@ cm_Analyze(cm_conn_t *connp,
             osi_Log3(afsd_logp, format, osi_LogSaveString(afsd_logp,addr), fidp->volume, cellp->name);
             LogEvent(EVENTLOG_WARNING_TYPE, msgID, addr, fidp->volume, cellp->name);
 
-            cm_SetServerBusyStatus(volServerspp, serverp);
+            if (volServerspp)
+                cm_SetServerBusyStatus(volServerspp, serverp);
         }
 
         if (free_svr_list) {
@@ -832,6 +835,12 @@ cm_Analyze(cm_conn_t *connp,
             reqp->vnovolError++;
         }
 
+        /* Remember that the VIO error occurred */
+        if (errorCode == VIO) {
+            reqp->errorServp = serverp;
+            reqp->vioCount++;
+        }
+
         /* Free the server list before cm_ForceUpdateVolume is called */
         if (free_svr_list) {
             cm_FreeServerList(volServerspp, 0);
@@ -839,7 +848,7 @@ cm_Analyze(cm_conn_t *connp,
             free_svr_list = 0;
         }
 
-        if ( timeLeft > 2 )
+        if ( timeLeft > 2 && reqp->vioCount < 100)
             retry = 1;
     } else if ( errorCode == VNOVNODE ) {
 	if (connp)
