@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,21 +10,17 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#endif
 #include <rx/xdr.h>
 #include <afs/vlserver.h>	/*Misc server-side Volume Location stuff */
 #include <ubik.h>
-#include "volser.h"
+#include <afs/volser.h>
+
 #include "bc.h"
 #include <afs/volint.h>
+#include <afs/volser.h>
+#include <afs/volser_prototypes.h>
 #include <afs/com_err.h>
 
 extern char *whoami;
@@ -34,7 +30,7 @@ extern char *whoami;
 /* ********************************************************************* */
 
 afs_int32
-bc_GetEntryByID(struct ubik_client *uclient, afs_int32 volID, 
+bc_GetEntryByID(struct ubik_client *uclient, afs_int32 volID,
 		afs_int32 volType, struct vldbentry *vldbEntryPtr)
 {
     afs_int32 code = 0;
@@ -55,7 +51,7 @@ bc_GetEntryByID(struct ubik_client *uclient, afs_int32 volID,
  */
 
 afs_int32
-volImageTime(afs_int32 serv, afs_int32 part, afs_int32 volid, 
+volImageTime(afs_uint32 serv, afs_int32 part, afs_uint32 volid,
 	     afs_int32 voltype, afs_int32 *clDatePtr)
 {
     afs_int32 code = 0;
